@@ -14,13 +14,14 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2019 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2020 Live Networks, Inc.  All rights reserved.
 // A data structure that represents a session that consists of
 // potentially multiple (audio and/or video) sub-sessions
 // Implementation
 
 #include "liveMedia.hh"
 #include "Locale.hh"
+#include "Base64.hh"
 #include "GroupsockHelper.hh"
 #include <ctype.h>
 
@@ -62,7 +63,8 @@ MediaSession::MediaSession(UsageEnvironment& env)
     fConnectionEndpointName(NULL),
     fMaxPlayStartTime(0.0f), fMaxPlayEndTime(0.0f), fAbsStartTime(NULL), fAbsEndTime(NULL),
     fScale(1.0f), fSpeed(1.0f),
-    fMediaSessionType(NULL), fSessionName(NULL), fSessionDescription(NULL), fControlPath(NULL) {
+    fMediaSessionType(NULL), fSessionName(NULL), fSessionDescription(NULL), fControlPath(NULL),
+    fMIKEYState(NULL), fCrypto(NULL) {
   fSourceFilterAddr.s_addr = 0;
 
   // Get our host name, and use this for the RTCP CNAME:
@@ -87,6 +89,7 @@ MediaSession::~MediaSession() {
   delete[] fSessionName;
   delete[] fSessionDescription;
   delete[] fControlPath;
+  delete fCrypto; delete fMIKEYState;
 }
 
 Boolean MediaSession::isMediaSession() const {
@@ -119,6 +122,7 @@ Boolean MediaSession::initializeWithSDP(char const* sdpDescription) {
     if (parseSDPAttribute_range(sdpLine)) continue;
     if (parseSDPAttribute_type(sdpLine)) continue;
     if (parseSDPAttribute_source_filter(sdpLine)) continue;
+    if (parseSDPAttribute_key_mgmt(sdpLine)) continue;
   }
 
   while (sdpLine != NULL) {
@@ -129,8 +133,8 @@ Boolean MediaSession::initializeWithSDP(char const* sdpDescription) {
       return False;
     }
 
-    // Parse the line as "m=<medium_name> <client_portNum> RTP/AVP <fmt>"
-    // or "m=<medium_name> <client_portNum>/<num_ports> RTP/AVP <fmt>"
+    // Parse the line as "m=<medium_name> <client_portNum> <proto> <fmt>"
+    // or "m=<medium_name> <client_portNum>/<num_ports> <proto> <fmt>"
     // (Should we be checking for >1 payload format number here?)#####
     char* mediumName = strDupSize(sdpLine); // ensures we have enough space
     char const* protocolName = NULL;
@@ -141,6 +145,12 @@ Boolean MediaSession::initializeWithSDP(char const* sdpDescription) {
 		mediumName, &subsession->fClientPortNum, &payloadFormat) == 3)
 	&& payloadFormat <= 127) {
       protocolName = "RTP";
+    } else if ((sscanf(sdpLine, "m=%s %hu RTP/SAVP %u",
+		       mediumName, &subsession->fClientPortNum, &payloadFormat) == 3 ||
+		sscanf(sdpLine, "m=%s %hu/%*u RTP/SAVP %u",
+		       mediumName, &subsession->fClientPortNum, &payloadFormat) == 3)
+	       && payloadFormat <= 127) {
+      protocolName = "SRTP";
     } else if ((sscanf(sdpLine, "m=%s %hu UDP %u",
 		       mediumName, &subsession->fClientPortNum, &payloadFormat) == 3 ||
 		sscanf(sdpLine, "m=%s %hu udp %u",
@@ -213,6 +223,7 @@ Boolean MediaSession::initializeWithSDP(char const* sdpDescription) {
       if (subsession->parseSDPAttribute_source_filter(sdpLine)) continue;
       if (subsession->parseSDPAttribute_x_dimensions(sdpLine)) continue;
       if (subsession->parseSDPAttribute_framerate(sdpLine)) continue;
+      if (subsession->parseSDPAttribute_key_mgmt(sdpLine)) continue;
 
       // (Later, check for malformed lines, and other valid SDP lines#####)
     }
@@ -275,87 +286,88 @@ Boolean MediaSession::parseSDPLine(char const* inputLine,
   return True;
 }
 
-static char* parseCLine(char const* sdpLine) {
-  char* resultStr = NULL;
-  char* buffer = strDupSize(sdpLine); // ensures we have enough space
-  if (sscanf(sdpLine, "c=IN IP4 %[^/\r\n]", buffer) == 1) {
-    // Later, handle the optional /<ttl> and /<numAddresses> #####
-    resultStr = strDup(buffer);
+// Common code used to parse many string values within SDP lines:
+static Boolean parseStringValue(char const* sdpLine, char const* searchFormat, char*& result) {
+  Boolean parseSuccess = False;
+  char* buffer = strDupSize(sdpLine);
+
+  if (sscanf(sdpLine, searchFormat, buffer) == 1) {
+    delete[] result; result = strDup(buffer);
+    parseSuccess = True;
   }
   delete[] buffer;
 
-  return resultStr;
+  return parseSuccess;
+}
+
+static Boolean parseTwoStringValues(char const* sdpLine, char const* searchFormat,
+				    char*& result1, char*& result2) {
+  Boolean parseSuccess = False;
+  size_t sdpLineSize = strlen(sdpLine) + 1;
+  char* buffer1 = new char[sdpLineSize];
+  char* buffer2 = new char[sdpLineSize];
+
+  if (sscanf(sdpLine, searchFormat, buffer1, buffer2) == 2) {
+    delete[] result1; result1 = strDup(buffer1);
+    delete[] result2; result2 = strDup(buffer2);
+    parseSuccess = True;
+  }
+  delete[] buffer1;
+  delete[] buffer2;
+
+  return parseSuccess;
+}
+
+static MIKEYState* parseSDPAttribute_key_mgmtToMIKEY(char const* sdpLine) {
+  char* keyMgmtPrtclId = NULL;
+  char* keyMgmtData = NULL;
+  MIKEYState* resultMIKEYState = NULL;
+
+  do {
+    // Check for a "a=key-mgmt:<prtcl-id> <keymgmt-data>" line:
+    if (!parseTwoStringValues(sdpLine, "a=key-mgmt:%s %s", keyMgmtPrtclId, keyMgmtData)) break;
+
+    // We understand only the 'protocol id' "mikey":
+    if (strcmp(keyMgmtPrtclId, "mikey") != 0) break;
+
+    // Base64-decode the "keyMgmtData" string:
+    unsigned keyMgmtData_decodedSize;
+    u_int8_t* keyMgmtData_decoded = base64Decode(keyMgmtData, keyMgmtData_decodedSize);
+    if (keyMgmtData_decoded == NULL) break;
+
+    resultMIKEYState = MIKEYState::createNew(keyMgmtData_decoded, keyMgmtData_decodedSize);
+  } while (0);
+
+  delete[] keyMgmtPrtclId;
+  delete[] keyMgmtData;
+  return resultMIKEYState;
 }
 
 Boolean MediaSession::parseSDPLine_s(char const* sdpLine) {
   // Check for "s=<session name>" line
-  char* buffer = strDupSize(sdpLine);
-  Boolean parseSuccess = False;
-
-  if (sscanf(sdpLine, "s=%[^\r\n]", buffer) == 1) {
-    delete[] fSessionName; fSessionName = strDup(buffer);
-    parseSuccess = True;
-  }
-  delete[] buffer;
-
-  return parseSuccess;
+  return parseStringValue(sdpLine, "s=%[^\r\n]", fSessionName);
 }
 
 Boolean MediaSession::parseSDPLine_i(char const* sdpLine) {
   // Check for "i=<session description>" line
-  char* buffer = strDupSize(sdpLine);
-  Boolean parseSuccess = False;
-
-  if (sscanf(sdpLine, "i=%[^\r\n]", buffer) == 1) {
-    delete[] fSessionDescription; fSessionDescription = strDup(buffer);
-    parseSuccess = True;
-  }
-  delete[] buffer;
-
-  return parseSuccess;
+  return parseStringValue(sdpLine, "i=%[^\r\n]", fSessionDescription);
 }
 
 Boolean MediaSession::parseSDPLine_c(char const* sdpLine) {
   // Check for "c=IN IP4 <connection-endpoint>"
   // or "c=IN IP4 <connection-endpoint>/<ttl+numAddresses>"
   // (Later, do something with <ttl+numAddresses> also #####)
-  char* connectionEndpointName = parseCLine(sdpLine);
-  if (connectionEndpointName != NULL) {
-    delete[] fConnectionEndpointName;
-    fConnectionEndpointName = connectionEndpointName;
-    return True;
-  }
-
-  return False;
+  return parseStringValue(sdpLine, "c=IN IP4 %[^/\r\n]", fConnectionEndpointName);
 }
 
 Boolean MediaSession::parseSDPAttribute_type(char const* sdpLine) {
   // Check for a "a=type:broadcast|meeting|moderated|test|H.332|recvonly" line:
-  Boolean parseSuccess = False;
-
-  char* buffer = strDupSize(sdpLine);
-  if (sscanf(sdpLine, "a=type: %[^ ]", buffer) == 1) {
-    delete[] fMediaSessionType;
-    fMediaSessionType = strDup(buffer);
-    parseSuccess = True;
-  }
-  delete[] buffer;
-
-  return parseSuccess;
+  return parseStringValue(sdpLine, "a=type: %[^ ]", fMediaSessionType);
 }
 
 Boolean MediaSession::parseSDPAttribute_control(char const* sdpLine) {
   // Check for a "a=control:<control-path>" line:
-  Boolean parseSuccess = False;
-
-  char* controlPath = strDupSize(sdpLine); // ensures we have enough space
-  if (sscanf(sdpLine, "a=control: %s", controlPath) == 1) {
-    parseSuccess = True;
-    delete[] fControlPath; fControlPath = strDup(controlPath);
-  }
-  delete[] controlPath;
-
-  return parseSuccess;
+  return parseStringValue(sdpLine, "a=control: %s", fControlPath);
 }
 
 static Boolean parseRangeAttribute(char const* sdpLine, double& startTime, double& endTime) {
@@ -410,10 +422,9 @@ static Boolean parseSourceFilterAttribute(char const* sdpLine,
   // one of our multicast addresses.  We also don't support more than
   // one <source> #####
   Boolean result = False; // until we succeed
-  char* sourceName = strDupSize(sdpLine); // ensures we have enough space
+  char* sourceName = NULL;
   do {
-    if (sscanf(sdpLine, "a=source-filter: incl IN IP4 %*s %s",
-	       sourceName) != 1) break;
+    if (!parseStringValue(sdpLine, "a=source-filter: incl IN IP4 %*s %s", sourceName)) break;
 
     // Now, convert this name to an address, if we can:
     NetAddressList addresses(sourceName);
@@ -436,6 +447,17 @@ Boolean MediaSession
   return parseSourceFilterAttribute(sdpLine, fSourceFilterAddr);
 }
 
+Boolean MediaSession::parseSDPAttribute_key_mgmt(char const* sdpLine) {
+  MIKEYState* newMIKEYState = parseSDPAttribute_key_mgmtToMIKEY(sdpLine);
+  if (newMIKEYState == NULL) return False;
+
+  delete fCrypto; delete fMIKEYState;
+  fMIKEYState = newMIKEYState;
+  fCrypto = new SRTPCryptographicContext(*fMIKEYState);
+  
+  return True;
+}
+
 char* MediaSession::lookupPayloadFormat(unsigned char rtpPayloadType,
 					unsigned& freq, unsigned& nCh) {
   // Look up the codec name and timestamp frequency for known (static)
@@ -600,6 +622,7 @@ MediaSubsession::MediaSubsession(MediaSession& parent)
     fClientPortNum(0), fRTPPayloadFormat(0xFF),
     fSavedSDPLines(NULL), fMediumName(NULL), fCodecName(NULL), fProtocolName(NULL),
     fRTPTimestampFrequency(0), fMultiplexRTCPWithRTP(False), fControlPath(NULL),
+    fMIKEYState(NULL), fCrypto(NULL),
     fSourceFilterAddr(parent.sourceFilterAddr()), fBandwidth(0),
     fPlayStartTime(0.0), fPlayEndTime(0.0), fAbsStartTime(NULL), fAbsEndTime(NULL),
     fVideoWidth(0), fVideoHeight(0), fVideoFPS(0), fNumChannels(1), fScale(1.0f), fNPT_PTS_Offset(0.0f),
@@ -626,6 +649,7 @@ MediaSubsession::~MediaSubsession() {
   delete[] fConnectionEndpointName; delete[] fSavedSDPLines;
   delete[] fMediumName; delete[] fCodecName; delete[] fProtocolName;
   delete[] fControlPath;
+  delete fCrypto; delete fMIKEYState;
   delete[] fAbsStartTime; delete[] fAbsEndTime;
   delete[] fSessionId;
 
@@ -689,9 +713,11 @@ Boolean MediaSubsession::initiate(int useSpecialRTPoffset) {
     tempAddr.s_addr = connectionEndpointAddress();
         // This could get changed later, as a result of a RTSP "SETUP"
 
+    Boolean const useSRTP = strcmp(fProtocolName, "SRTP") == 0;
+    Boolean const protocolIsRTP = useSRTP || strcmp(fProtocolName, "RTP") == 0;
+
     if (fClientPortNum != 0 && (honorSDPPortChoice || IsMulticastAddress(tempAddr.s_addr))) {
       // The sockets' port numbers were specified for us.  Use these:
-      Boolean const protocolIsRTP = strcmp(fProtocolName, "RTP") == 0;
       if (protocolIsRTP && !fMultiplexRTCPWithRTP) {
 	fClientPortNum = fClientPortNum&~1;
 	    // use an even-numbered port for RTP, and the next (odd-numbered) port for RTCP
@@ -820,6 +846,19 @@ Boolean MediaSubsession::initiate(int useSpecialRTPoffset) {
       env().setResultMsg("Failed to create read source");
       break;
     }
+    
+    SRTPCryptographicContext* ourCrypto = NULL;
+    if (useSRTP) {
+      // For SRTP, we need key management.  If MIKEY (key management) state wasn't given
+      // to us in the SDP description, then create it now:
+      ourCrypto = getCrypto();
+      if (ourCrypto == NULL) { // then fMIKEYState is also NULL; create both
+	fMIKEYState = new MIKEYState();
+	ourCrypto = fCrypto = new SRTPCryptographicContext(*fMIKEYState);
+      }
+
+      if (fRTPSource != NULL) fRTPSource->setCrypto(ourCrypto);
+    }
 
     // Finally, create our RTCP instance. (It starts running automatically)
     if (fRTPSource != NULL && fRTCPSocket != NULL) {
@@ -832,7 +871,9 @@ Boolean MediaSubsession::initiate(int useSpecialRTPoffset) {
 					      (unsigned char const*)
 					      fParent.CNAME(),
 					      NULL /* we're a client */,
-					      fRTPSource);
+					      fRTPSource,
+					      False /* we're not a data transmitter */,
+					      ourCrypto);
       if (fRTCPInstance == NULL) {
 	env().setResultMsg("Failed to create RTCP instance");
 	break;
@@ -997,14 +1038,7 @@ Boolean MediaSubsession::parseSDPLine_c(char const* sdpLine) {
   // Check for "c=IN IP4 <connection-endpoint>"
   // or "c=IN IP4 <connection-endpoint>/<ttl+numAddresses>"
   // (Later, do something with <ttl+numAddresses> also #####)
-  char* connectionEndpointName = parseCLine(sdpLine);
-  if (connectionEndpointName != NULL) {
-    delete[] fConnectionEndpointName;
-    fConnectionEndpointName = connectionEndpointName;
-    return True;
-  }
-
-  return False;
+  return parseStringValue(sdpLine, "c=IN IP4 %[^/\r\n]", fConnectionEndpointName);
 }
 
 Boolean MediaSubsession::parseSDPLine_b(char const* sdpLine) {
@@ -1060,16 +1094,7 @@ Boolean MediaSubsession::parseSDPAttribute_rtcpmux(char const* sdpLine) {
 
 Boolean MediaSubsession::parseSDPAttribute_control(char const* sdpLine) {
   // Check for a "a=control:<control-path>" line:
-  Boolean parseSuccess = False;
-
-  char* controlPath = strDupSize(sdpLine); // ensures we have enough space
-  if (sscanf(sdpLine, "a=control: %s", controlPath) == 1) {
-    parseSuccess = True;
-    delete[] fControlPath; fControlPath = strDup(controlPath);
-  }
-  delete[] controlPath;
-
-  return parseSuccess;
+  return parseStringValue(sdpLine, "a=control: %s", fControlPath);
 }
 
 Boolean MediaSubsession::parseSDPAttribute_range(char const* sdpLine) {
@@ -1180,6 +1205,17 @@ Boolean MediaSubsession::parseSDPAttribute_framerate(char const* sdpLine) {
   return parseSuccess;
 }
 
+Boolean MediaSubsession::parseSDPAttribute_key_mgmt(char const* sdpLine) {
+  MIKEYState* newMIKEYState = parseSDPAttribute_key_mgmtToMIKEY(sdpLine);
+  if (newMIKEYState == NULL) return False;
+
+  delete fCrypto; delete fMIKEYState;
+  fMIKEYState = newMIKEYState;
+  fCrypto = new SRTPCryptographicContext(*fMIKEYState);
+  
+  return True;
+}
+
 Boolean MediaSubsession::createSourceObjects(int useSpecialRTPoffset) {
   do {
     // First, check "fProtocolName"
